import typer
from tabulate import tabulate

import dynamic_solvers
import functions
import static_solvers

STATIC_METHODS = [
    static_solvers.left_rectangle_method,
    static_solvers.right_rectangle_method,
    static_solvers.trapezium_method,
    static_solvers.parabola_method,
]
DYNAMIC_METHODS = [
    dynamic_solvers.double_recalculate_method,
    dynamic_solvers.runge_kutty_method,
    dynamic_solvers.euler_method,
    dynamic_solvers.mysterious_method,
]
ALL_METHODS = [
    *STATIC_METHODS,
    *DYNAMIC_METHODS,
]

ALL_FUNCTIONS = [
    functions.fn_sin,
    functions.fn_cos,
    functions.fn_ln,
    functions.fn_sqrt,
    functions.fn_square,
    functions.fn_cube,
]


def list_methods() -> None:
    """вывести все доступные методы"""
    headers = ["№", "Название метода"]
    method_names = [method.__doc__ for method in ALL_METHODS]
    method_table = tabulate(
        tabular_data=[[no, name] for no, name in enumerate(method_names, start=1)],
        headers=headers,
    )
    typer.echo("Доступные методы с постоянным и переменным шагом:")
    typer.echo(method_table + "\n")


def list_functions() -> None:
    """вывести все доступные функции"""
    headers = ["№", "Функция"]
    function_names = [function.__doc__ for function in ALL_FUNCTIONS]
    function_table = tabulate(
        tabular_data=[[no, name] for no, name in enumerate(function_names, start=1)],
        headers=headers,
    )
    typer.echo("Доступные функции для интегрирования:")
    typer.echo(function_table + "\n")


def main() -> None:
    """интерактивный консольный интерфейс"""
    typer.echo("ЛР1: Система решения ОДУ")
    typer.echo("Работу выполнили: Величко А.А., Галкин И.Ю., Егоров С.А., Адаев Р.М. ИВТ 2 курс\n")

    # вывести все доступные методы
    list_methods()

    # ввести номер необходимого метода
    while True:
        method_num: int = typer.prompt("Выберите метод из списка и введите его номер", type=int)
        if 1 <= method_num <= len(ALL_METHODS):
            break
        typer.echo(f"Номера {method_num} нет в списке")

    method = ALL_METHODS[method_num - 1]
    typer.echo(f"Выбран метод: {method.__doc__}")

    # вывести все доступные функции
    list_functions()

    # ввести номер необходимой функции
    while True:
        function_num: int = typer.prompt("Выберите функцию из списка и введите её номер", type=int)
        if 1 <= function_num <= len(ALL_FUNCTIONS):
            break
        typer.echo(f"Номера {function_num} нет в списке")

    function = ALL_FUNCTIONS[function_num - 1]
    typer.echo(f"Выбрана функция: {function.__doc__}")

    # ввести аргументы
    a: float = typer.prompt("Нижний предел интегрирования", type=float)
    b: float = typer.prompt("Верхний предел интегрирования", type=float)
    n: float = typer.prompt("Кол-во разбиений", type=int)

    # произвести вычисления и вывести результат
    result: float = method(function, a, b, n)
    typer.echo(f"Результат вычислений: {result}\n")


if __name__ == "__main__":
    typer.run(main)
