import typing as tp


def left_rectangle_method(fx: tp.Callable[[float], float], a: float, b: float, steps: int) -> float:
    """Метод левых прямоугольников"""
    h = (b - a) / steps
    x = a
    area = 0.0

    for _ in range(steps):
        area += h * fx(x)
        x += h

    return area


def right_rectangle_method(
    fx: tp.Callable[[float], float], a: float, b: float, steps: int
) -> float:
    """Метод правых прямоугольников"""
    h = (b - a) / steps
    x = a + h
    area = 0.0

    for _ in range(steps):
        area += h * fx(x)
        x += h

    return area


def trapezium_method(fx: tp.Callable[[float], float], a: float, b: float, steps: int) -> float:
    """Метод трапеций"""
    h: float = (b - a) / steps
    ans: float = 0.0
    x: float = a

    for _ in range(steps):
        ans += h * (fx(x) + fx(x + h)) / 2
        x += h

    return ans


def parabola_method(fx: tp.Callable[[float], float], a: float, b: float, steps: int) -> float:
    """Метод парабол"""
    if steps % 2 == 1:
        steps += 1

    h: float = (b - a) / steps
    summa: float = fx(a) + 4 * fx(a + h) + fx(b)

    for i in range(1, int(steps / 2)):
        summa += 2 * fx(a + (2 * i) * h) + 4 * fx(a + (2 * i + 1) * h)

    return summa * h / 3
